@@ -43,6 +43,10 @@ from The Open Group.
 #include <X11/Xproto.h>		/* to declare xEvent */
 #include <X11/XlibConf.h>	/* for configured options like XTHREADS */
 
+#ifdef XTHREADS
+#include <X11/Xthreads.h>
+#endif
+
 /* The Xlib structs are full of implicit padding to properly align members.
    We can't clean that up without breaking ABI, so tell clang not to bother
    complaining about it. */
@@ -207,6 +211,10 @@ struct _XDisplay
 
 	XIOErrorExitHandler exit_handler;
 	void *exit_handler_data;
+	CARD32 in_ifevent;
+#ifdef XTHREADS
+    xthread_t ifevent_thread;
+#endif
 };
 
 #define XAllocIDs(dpy,ids,n) (*(dpy)->idlist_alloc)(dpy,ids,n)
@@ -581,11 +589,11 @@ extern void *_XGetRequest(Display *dpy, CARD8 type, size_t len);
 #define MakeBigReq(req,n) \
     { \
     CARD64 _BRdat; \
-    CARD32 _BRlen = req->length - 1; \
+    CARD32 _BRlen = (CARD32) (req->length - 1); \
     req->length = 0; \
     _BRdat = ((CARD32 *)req)[_BRlen]; \
     memmove(((char *)req) + 8, ((char *)req) + 4, (_BRlen - 1) << 2); \
-    ((CARD32 *)req)[1] = _BRlen + n + 2; \
+    ((CARD32 *)req)[1] = _BRlen + (CARD32) (n) + 2; \
     Data32(dpy, &_BRdat, 4); \
     }
 #else
@@ -596,7 +604,7 @@ extern void *_XGetRequest(Display *dpy, CARD8 type, size_t len);
     req->length = 0; \
     _BRdat = ((CARD32 *)req)[_BRlen]; \
     memmove(((char *)req) + 8, ((char *)req) + 4, (_BRlen - 1) << 2); \
-    ((CARD32 *)req)[1] = _BRlen + n + 2; \
+    ((CARD32 *)req)[1] = _BRlen + (CARD32) (n) + 2; \
     Data32(dpy, &_BRdat, 4); \
     }
 #endif
@@ -615,10 +623,10 @@ extern void *_XGetRequest(Display *dpy, CARD8 type, size_t len);
 	    MakeBigReq(req,n) \
 	} else { \
 	    n = badlen; \
-	    req->length += n; \
+	    req->length = (CARD16) (req->length + n); \
 	} \
     } else \
-	req->length += n
+	req->length = (CARD16) (req->length + n)
 #else
 #define SetReqLen(req,n,badlen) \
     req->length += n
@@ -641,10 +649,10 @@ extern void _XFlushGCCache(Display *dpy, GC gc);
 #ifndef DataRoutineIsProcedure
 #define Data(dpy, data, len) {\
 	if (dpy->bufptr + (len) <= dpy->bufmax) {\
-		memcpy(dpy->bufptr, data, (int)(len));\
-		dpy->bufptr += ((len) + 3) & ~3;\
+		memcpy(dpy->bufptr, data, (size_t)(len));\
+		dpy->bufptr += ((size_t)((len) + 3) & (size_t)~3);\
 	} else\
-		_XSend(dpy, data, len);\
+		_XSend(dpy, (_Xconst char*)(data), (long)(len));\
 }
 #endif /* DataRoutineIsProcedure */
 
@@ -667,14 +675,14 @@ extern void _XFlushGCCache(Display *dpy, GC gc);
     if (dpy->bufptr + (n) > dpy->bufmax) \
         _XFlush (dpy); \
     ptr = (type) dpy->bufptr; \
-    memset(ptr, '\0', n); \
+    memset(ptr, '\0', (size_t)(n)); \
     dpy->bufptr += (n);
 
 #define Data16(dpy, data, len) Data((dpy), (_Xconst char *)(data), (len))
 #define _XRead16Pad(dpy, data, len) _XReadPad((dpy), (char *)(data), (len))
 #define _XRead16(dpy, data, len) _XRead((dpy), (char *)(data), (len))
 #ifdef LONG64
-#define Data32(dpy, data, len) _XData32(dpy, (_Xconst long *)data, len)
+#define Data32(dpy, data, len) _XData32(dpy, (_Xconst long *)(data), (unsigned)(len))
 extern int _XData32(
 	     Display *dpy,
 	     _Xconst long *data,
@@ -686,7 +694,7 @@ extern void _XRead32(
 	     long len
 );
 #else
-#define Data32(dpy, data, len) Data((dpy), (_Xconst char *)(data), (len))
+#define Data32(dpy, data, len) Data((dpy), (_Xconst char *)(data), (long)(len))
 #define _XRead32(dpy, data, len) _XRead((dpy), (char *)(data), (len))
 #endif
 
